clear all; clc;
addpath(genpath('functions'))
disp(datestr(now))
set(0,'defaulttextInterpreter','latex')

%% Plot settings
lwd = 1; %linewidth
fnt = 11; %FontSize

%% Controls
N_vec = round(2.^(3:0.5:10));
% Remark: later, a large N for warmup purposes is added to N_vec_0

%% NLDE parameters, Initial data, potentials, discretization, ...
pars.ep = 0.01;
pars.a = -16; pars.b = 16; % domain [a,b]
pars.gamma_1 = 0; % beta-nonlinearity (gamma_1 =/= 0 not implemented)
pars.gamma_2 = 1; % norm-nonlinearity
pars.t_end = 1;
% electric potential:
pars.V_func = @(x) (1-x)./(1+x.^2); pars.name_V = 'Bao3';
% magnetic potential:
pars.A_1_func = @(x) (x+1).^2./(1+x.^2); pars.name_A_1 = 'Bao4'; 
% initial data (two components):
pars.psi_start_1_func = @(x) exp(-x.^2/2); pars.name_psi_start_1 = 'Bao1';
pars.psi_start_2_func = @(x) exp(-(x-1).^2/2); pars.name_psi_start_2 = 'Bao2';

pars.M = 128; % Divide [a,b] in 2M discretization points x_{-M}, ..., x_{M-1}
pars.m=1; % For Sobolev_norm
pars.tau = 0.1; % variable only need for technical reasons, value is irrelevant

% How many starting steps to compute first approximation in two step
% method?
pars.num_start_steps = 3;
% Only even step numbers?
force_even = true;
if force_even
    N_vec( mod(N_vec,2) == 1 ) = N_vec( mod(N_vec,2) == 1 ) + 1;
end
% tolerances for reference solutions
rel_tol_limit = 1e-12;
abs_tol_limit = 1e-14;

methods = {'benchmark','EEMR'};

%% create all necessary data from parameters above
[pars] = create_discretized_data(pars);
% define new base folder
pars = create_base_folder(pars);

%% User controled options
disp('Methods are applied multiple times and runtime is averaged.')
disp('How many runs do you want? (Preprint: 5 runs, total runtime < 1 min)')
number_of_runs = input('Selection: ');

%% Check if results exist already. If yes, then ask user what to do
filename = [pars.base_fld,'/timed solutions/',...
            num2str(number_of_runs),' runs (',...
            num2str(pars.num_start_steps),' start steps in EEMR) .mat']';
selection = 0;
if exist(filename,'file')>0
    disp('Efficieny of methods in desired settings has already been analyzed before.')
    disp('Time methods again or show results from previous run?')
    disp('Time methods again    --> press 0')
    disp('Show previous results --> press 1')
    selection = input('Selection: ');
elseif exist([pars.base_fld,'/timed solutions'],'dir') == 0
    mkdir([pars.base_fld,'/timed solutions'])
end

%% Create Figure and tiled layout
fig = figure('Units','points','Position',[100,100,500,240]);
fig_laout = tiledlayout(1,2,'Padding','Tight','TileSpacing','tight');

%% Load or compute results
if selection == 1 % load results
    load(filename)
else % compute results
    %% Preparations
    % compute or load reference solution of limit system
    psi_ref = compute_or_load_ref_solution('limit system',rel_tol_limit,abs_tol_limit,pars);
    % Data structure to save all measured times (for each N, each method
    % and each run)
    time_mat    = zeros(length(N_vec),length(methods),number_of_runs);
    fft_cnt_mat = zeros(length(N_vec),length(methods),number_of_runs);
    err_mat     = zeros(length(N_vec),length(methods),number_of_runs);
    %% Start Runs
    for run = 1:number_of_runs  
        disp(['Run ' num2str(run) ' of ' num2str(number_of_runs)])            
        % for a warmup, compute solutions for a relatively large N first, but
        % don't include it in the plot
        N_vec_ext = [2000,N_vec];
        % stepsizes in random order
        indices = randperm(length(N_vec));
        % Calculate solution for every N
        for ind = [1,indices+1]
            N = N_vec_ext(ind);
            pars.tau = pars.t_end/N;
            % is this the warmup run?
            if ind == 1
                disp('%%%%% Warmup %%%%%%')
            end
            for met_ind = 1:length(methods)
                method = methods{met_ind};
                disp([method,' method for N = ',num2str(N),' is applied.'])
                [psi_limit,time,fft_count] = compute_method_approx(method,N,pars);
                if ind ~= 1 % save results except for the warump
                    time_mat(ind-1,met_ind,run) = time;
                    fft_cnt_mat(ind-1,met_ind,run) = fft_count;
                    err_mat(ind-1,met_ind,run) = sobolev_norm(psi_limit-psi_ref,pars.m,pars.M,pars.a,pars.b);
                end
                disp([method,' method for N = ',num2str(N),' has been applied, time ',num2str(time),'.'])
            end % end methods
            if ind == 1
                disp('%%%%% Warmup over %%%%%%')
            end
        end % end N_vec
    end % end runs
     
    %% Takes averages
    time_mat_avg    = sum(time_mat,3) / number_of_runs;
    fft_cnt_mat_avg = sum(fft_cnt_mat,3) / number_of_runs;
    err_mat_avg     = sum(err_mat,3) / number_of_runs;
    %% Save results
    save(filename,'time_mat_avg','fft_cnt_mat_avg','err_mat_avg')
end % end compute or load

%% Create plot
% Compute limits of time/fft_cnt/error
min_time = min(min(time_mat_avg));
max_time = max(max(time_mat_avg));
min_fft_cnt = min(min(fft_cnt_mat_avg));
max_fft_cnt = max(max(fft_cnt_mat_avg));
min_err = min(min(err_mat_avg));
max_err = max(max(err_mat_avg));
% Add error line for each method and for both subplots
styles = {'*-','*--'};
for met_ind = 1:length(methods)
    method = methods{met_ind};
    nexttile(1) % error over time
    set(gca,'ColorOrderIndex',1)
    loglog(time_mat_avg(:,met_ind),err_mat_avg(:,met_ind),styles{met_ind},'DisplayName',method,'LineWidth',lwd)
    hold on
    nexttile(2) % error over fft count
    set(gca,'ColorOrderIndex',1)
    loglog(fft_cnt_mat_avg(:,met_ind),err_mat_avg(:,met_ind),styles{met_ind},'DisplayName',method,'LineWidth',lwd)
    hold on
end
% Compute constant of and plot reference line 
est_error_constants = zeros(length(methods),2);
for met_ind = 1:length(methods)
    % Estimate error constant: err = C * 1 / time^2 
    % --> log(err) = log(C) - 2 * log(time) --> log(err) + 2 * log(time) = log(C)
    % --> C = exp( log(err) + 2*log(time) )
    % Compute exponential of mean of log(err) + 2 * log(time)
    % Estimated constant error over time
    est_error_constants(met_ind,1) ...
        = exp( mean(log(err_mat_avg(:,met_ind)) + 2 * log(time_mat_avg(:,met_ind))) );
    % Estimated constant error over time
    est_error_constants(met_ind,2)  ...
        = exp( mean(log(err_mat_avg(:,met_ind)) + 2 * log(fft_cnt_mat_avg(:,met_ind))) );
end
% Plot reference lines and other plot settings
for i = 1:2 % plot over time and over fft count
    nexttile(i)
    % reference lines
    for met_ind = 1:length(methods)
        C = est_error_constants(met_ind,i);
        if i == 1 % over time
            C_label = num2str(C,3);
            C_label = strsplit(C_label, {'e-'});
            C_label = [C_label{1},'\cdot10^{-',C_label{2},'}'];
            loglog([1e-3,1],C*[1e-3,1].^(-2),...
                    styles{met_ind},'Color','k','Marker','none',...
                    'DisplayName',['line $',C_label,' \, t^{-2}$'])
        else % over fft count
            C_label = num2str(C,3);
            loglog([1,1e+4],C*[1,1e+4].^(-2),...
                    styles{met_ind},'Color','k','Marker','none',...
                    'DisplayName',['line $',num2str(C,3),' \, (\#FFT)^{-2}$'])
        end
    end
    % other plot settings
    if i == 1 % over time
        xlabel('computation time in sec','FontSize',fnt)
        ylabel(['error at time $t_{end}$ in $H^' num2str(pars.m) '$-norm'],'FontSize',fnt)
        xlim( [min_time,max_time] )
    else % over fft count
        xlabel('number of FFTs \& IFFTs','FontSize',fnt)
        yticklabels({'.','.'});
        xlim( [min_fft_cnt,max_fft_cnt] )
        ylim( [min_err,max_err] )
    end
    ylim( [min_err,max_err] )
    legend('Location','Northeast','FontSize',fnt,'Interpreter','latex');
end

%%  
rmpath(genpath('functions'))
disp(datestr(now))

%% save images
if false
    saveas(fig1,'BaoDataErrorOverRuntime.eps','epsc')
    saveas(fig1,'BaoDataErrorOverRuntime')
end
