clear all; 
addpath(genpath('functions'))
disp(datestr(now))

%% Plot setting
lwd = 1; % LineWidth
fnt = 11; % FontSize
y_limits = [4e-8,6e-2];

%% NLDE parameters, Initial data, potentials, discretization, ...
pars.a = -16; pars.b = 16; % domain [a,b]
pars.gamma_1 = 0; % beta-nonlinearity (gamma_1 =/= 0 not implemented)
pars.gamma_2 = 1; % norm-nonlinearity
pars.t_end = 1;
% electric potential:
pars.V_func = @(x) (1-x)./(1+x.^2); pars.name_V = 'Bao3';
% magnetic potential:
pars.A_1_func = @(x) (x+1).^2./(1+x.^2); pars.name_A_1 = 'Bao4'; 
% initial data (two components):
pars.psi_start_1_func = @(x) exp(-x.^2/2); pars.name_psi_start_1 = 'Bao1';
pars.psi_start_2_func = @(x) exp(-(x-1).^2/2); pars.name_psi_start_2 = 'Bao2';

pars.M = 128; % Divide [a,b] in 2M discretization points x_{-M}, ..., x_{M-1}
pars.m=1; % For Sobolev_norm
pars.tau = 0.1; % variable only need for technical reasons, value is irrelevant
% How many starting steps to compute first approximation in two step
% method?
pars.num_start_steps = 3;
% Only even step numbers?
force_even = true;

% tolerances for reference solutions
rel_tol = 1e-8;
abs_tol = 1e-10;
rel_tol_limit = 1e-12;
abs_tol_limit = 1e-14;

%% Ask for computation settings
% Which eps values?
disp('Which epsilon values?')
disp('Testing purposes: Only one epsilon value (0.01)      --> Press 0 (ca. 2min)')
disp('Paper: Multiple epsilon values (0.01, 0.005, 0.0025) --> Press 1 (ca. 15-20min)')
selection = input('Selection: ');
if selection == 0
    eps_vec = 0.01;
elseif selection == 1
    eps_vec = [0.0025,0.005,0.01];
else
    error('No valid selection')
end
% Which tau values (determined by the step number N)?
disp('Which step numbers?')
disp('Testing purposes: Coarse resolution (N = 2^3, ..., 2^10) --> Press 0')
disp('Paper: Fine resolution (N = 2^3, 2^3.5, ..., 2^12)       --> Press 1')
selection = input('Selection: ');
if selection == 0
    N_vec = round(2.^(3:10));
elseif selection == 1
    N_vec = round(2.^(3:0.5:12));
else
    error('No valid selection')
end
% Replace odd step numbers by even step numbers
if force_even
    N_vec( mod(N_vec,2) == 1 ) = N_vec( mod(N_vec,2) == 1 ) + 1;
    N_vec = unique(N_vec);
end
tau_vec = pars.t_end ./ N_vec;

%% create all necessary data from parameters above
[pars] = create_discretized_data(pars);

%% Create Figure and tiled layout
fig1 = figure('Units','points','Position',[100,100,500,240]);
fig_laout = tiledlayout(1,2,'Padding','Tight','TileSpacing','tight');

%% Iterate over all eps values
for ep_ind = 1:length(eps_vec)
    % current parameter
    pars.ep = eps_vec(ep_ind);
    disp(['eps ',num2str(ep_ind),' of ',num2str(length(eps_vec)),': eps = ',num2str(pars.ep)])
    % create multiplication vectors to apply operators in Fourier space
    vecs = create_multiplication_vectors(pars);
    % define new base folder
    pars = create_base_folder(pars);

    %% Compute reference solutions to both equations
    % Load if existent, else compute it
    psi_ref_NLDE = compute_or_load_ref_solution('NLDE',rel_tol,abs_tol,pars);            
    % Load if existent, else compute it
    psi_ref_limit = compute_or_load_ref_solution('limit system',rel_tol_limit,abs_tol_limit,pars);            

    %% compute approximations with both methods
    methods = {'benchmark','EEMR'}; styles = {'*-','*--'}; plot_names = {'benchm.','EEMR'};
    for met_ind = 1:length(methods)
        err_vec_NLDE = zeros(length(N_vec),1);
        err_vec_limit = zeros(length(N_vec),1);
        % iterate over all N
        for ind = 1:length(N_vec)
            N = N_vec(ind);
            psi_met = compute_or_load_method_approx(methods{met_ind},N,pars);            
            % calculate error vs ref. solution of NLDE
            err_vec_NLDE(ind) = sobolev_norm(psi_met-psi_ref_NLDE,pars.m,pars.M,pars.a,pars.b);
            % calculate error vs ref. solution of limit system
            err_vec_limit(ind) = sobolev_norm(psi_met-psi_ref_limit,pars.m,pars.M,pars.a,pars.b);
        end

        %% Plot global error compared to limit system
        nexttile(1)
        set(gca,'ColorOrderIndex',ep_ind);
        loglog(tau_vec,err_vec_limit,styles{met_ind},...
               'DisplayName',['$\varepsilon = ',num2str(pars.ep),'$, ',plot_names{met_ind}],...
               'LineWidth',lwd)
        hold on
        %% Plot global error compared to NLDE
        nexttile(2)
        set(gca,'ColorOrderIndex',ep_ind);
        loglog(tau_vec,err_vec_NLDE,styles{met_ind},...
               'DisplayName',['$\varepsilon = ',num2str(pars.ep),'$, ',plot_names{met_ind}],...
               'LineWidth',lwd)
        hold on
    end
end

%% decoration of plot
% second-order reference line, further plot settings
C = 3e-1;
for ind = 1:2
    nexttile(ind)
    loglog(tau_vec,C*tau_vec.^2,'k--','DisplayName','$\mathcal{O}(\tau^2)$','LineWidth',lwd)
    xlim([min(tau_vec),max(tau_vec)])
    ylim(y_limits)
    xlabel('step size $\tau$','FontSize',fnt,'Interpreter','Latex')
    if ind == 1
        ylabel('error at time $t_{end}$ in $H^1$-norm','FontSize',fnt,'Interpreter','Latex')
        leg = legend('Location','Northwest','FontSize',fnt-2,'Interpreter','Latex');
        leg.ItemTokenSize = [30,18];
        title('semi-nonrelativistic limit system','FontWeight','normal','FontSize',fnt,'Interpreter','Latex')
    else
        title('Dirac equation','FontWeight','normal','FontSize',fnt,'Interpreter','Latex')
        yticklabels('.')
    end
end

% show eps markers in NLDE error plot
nexttile(2)
for ep_ind = 1:length(eps_vec)
    ep = eps_vec(ep_ind);
    loglog(0,0,'k-.','LineWidth',lwd,...
        'DisplayName','marker $\tau=\epsilon$')
    set(gca,'ColorOrderIndex',ep_ind)
    loglog([ep,ep],[1e-15,1e4],'-.','LineWidth',lwd)
end

%%
rmpath(genpath('functions'))
disp(datestr(now))

%% save images
if false
    saveas(fig1,'BaoDataGlobalError.eps','epsc')
    saveas(fig1,'BaoDataGlobalError')
end
