function [psi,psi_four] = timestep_EEMR(psi,psi_four,psi_old,psi_old_four,t_start,pars,vecs)
% Perform one timestep of the double VoC method

% Storage scheme:
% Psi (and all other vectors corresponding to two component functions) is a
% vector of size 8*M:
% Psi(    1:2*M) --> Psi^+_1 at the nodes x_(-M),...,x_(M-1)
% Psi(2*M+1:4*M) --> Psi^+_2 at the nodes x_(-M),...,x_(M-1)
% Psi(4*M+1:6*M) --> Psi^-_1 at the nodes x_(-M),...,x_(M-1)
% Psi(6*M+1:8*M) --> Psi^-_2 at the nodes x_(-M),...,x_(M-1)
% Thus, to be more precise, 
%   psi = (psi^+,psi^-)^T = (psi^+_1,psi^+_2,psi^-_1,psi^-_2)^T

% Two step method
% - t_start corresponds to the time t_n
% - psi is the approximation at time t_n
% - psi_old is the approximation from the previous step, thus at time
% t_{n-1}

M = size(psi,1)/8;

%% Apply potential and non-linearity
W_psi = potential_W(psi,pars);
g_0 = abs(psi(    1:2*M)).^2 + abs(psi(2*M+1:4*M)).^2 + ...
      abs(psi(4*M+1:6*M)).^2 + abs(psi(6*M+1:8*M)).^2;
g_0_psi  = repmat(g_0, 4,1) .* psi;

%% apply projectors
temp = fft_2C2( W_psi + pars.gamma_2 * g_0_psi );
proj_p_W_and_G_psi_four_plus  =  plus_projector_C2( temp(    1:4*M) ,pars,vecs.proj);
proj_m_W_and_G_psi_four_minus = minus_projector_C2( temp(4*M+1:8*M) ,pars,vecs.proj);

%% Compute integral I
I_1_four = -1i * [ repmat(vecs.kappa_2_pl(:,3),2,1) .* proj_p_W_and_G_psi_four_plus ; ...
                   repmat(vecs.kappa_2_mi(:,3),2,1) .* proj_m_W_and_G_psi_four_minus ];
               
%% Calculate psi_new = approximation after one more time step
% Calculate the linear part exp(mp 2i tau D) psi in Fourier space, add the
% integral (it is in Fourier space) and perform an inverse Fourier
% transform in the end
psi_four = [repmat(vecs.exp2D_minus,2,1); ...
            repmat(vecs.exp2D_plus, 2,1)].* psi_old_four ...
           + I_1_four;
psi = ifft_2C2(psi_four);

end

